//{ Semantic, Semantics } = require('./semantics.js')
//import { Semantic, Semantics } from './semantics.js'
Semantic = require('./semantics.js').Semantic;
Semantics = require('./semantics.js').Semantics;

describe('Semantic Tests', () => {
  let context;

  beforeEach( () => {
    context = {
      "amount": 10,
      "marker": "earn",
      "units": "dollars",
      "who": "joe"
    }
  });

  it('semantic match', () => {
    const matcher = ({objects, context}) => {
      return context.marker == 'earn';
    }
    const handler = ({objects, context}) => {}
    const semantic = new Semantic(matcher, handler);
    objects = {}
    expect(semantic.matches(objects, context)).toBeTruthy()
  });

  it('semantic no match', () => {
    const matcher = ({context}) => context.marker == 'earn23';
    const handler = ({objects, context}) => {}
    const semantic = new Semantic(matcher, handler);
    objects = {}
    expect(semantic.matches(objects, context)).not.toBeTruthy()
  });

  it('semantic apply objects variables', () => {
    const matcher = ({context}) => context.marker == 'earn';
    const handler = ({objects, context}) => {
      objects.who = context.who; 
      objects.earnings = context.amount
      objects.units = "dollars";
    };
    const semantic = new Semantic(matcher, handler);

    objects = {}
    original = {...context}
    context_prime = semantic.apply(objects, context);
    expect(context_prime).toStrictEqual(context)
    expect(context).toStrictEqual(original)
    expect(objects).toStrictEqual({ who: 'joe', earnings: 10, units: 'dollars' });
  });

  it('semantic apply context variables', () => {
    const matcher = ({context}) => context.marker == 'earn';
    const handler = ({objects, context}) => {
      context.who = 'Joseph';
    };
    const semantic = new Semantic(matcher, handler);

    objects = {}
    original = {...context}
    context_prime = semantic.apply(objects, context);
    expect(context_prime).not.toStrictEqual(context)
    expect(context).toStrictEqual(original)
    original.who = 'Joseph'
    expect(context_prime).toStrictEqual(original)
  });
})

describe('Semantics Tests', () => {
  const contexts = [
    {
      'marker': 'earns',
      'who': 'joe',
      'earns': 10,
      'units': 'dollars',
    },
    {
      'marker': 'earns',
      'isQuery': true,
      'who': 'joe',
      'for': 'earnings'
    }
  ]

  const semantics = [
    new Semantic( 
      ({objects, context}) => context.marker == 'earns' && context.isQuery, 
      ({objects, context}) => { context.marker = "response"; context.earnings = objects.earnings; context.units = objects.units; }
    ),
    new Semantic( 
      ({objects, context}) => context.marker == 'earns',
      ({objects, context}) => { 
        objects.who = context.who; objects.earnings = context.earns; objects.units = "dollars"; 
      }
    )
  ]

  it('apply one', () => {
    const s = new Semantics([semantics[1]]);
    const objects = {}
    const cs = s.apply(objects, [contexts[0]]);
    expect(objects).toStrictEqual({ who: 'joe', earnings: 10, units: 'dollars' });
  });

  it('apply two', () => {
    const s = new Semantics(semantics);
    const objects = {}
    const cs = s.apply(objects, contexts);
    expect(objects).toStrictEqual({ who: 'joe', earnings: 10, units: 'dollars' });
    expect(cs[0]).toStrictEqual({ 'marker': 'earns', 'who': 'joe', 'earns': 10, 'units': 'dollars', });
    expect(cs[1]).toStrictEqual({"earnings": 10, "for": "earnings", "isQuery": true, "marker": "response", "units": "dollars", "who": "joe",})
    expect(cs).toStrictEqual([
      { 'marker': 'earns', 'who': 'joe', 'earns': 10, 'units': 'dollars', },
      {"earnings": 10, "for": "earnings", "isQuery": true, "marker": "response", "units": "dollars", "who": "joe",}
    ])
  });

  it('apply one call s', () => {
    const s1 = new Semantic( 
      ({objects, context}) => context.marker == 'time', 
      ({objects, context}) => { 
        context.isResponse = true;
      }
    );
    const s2 = new Semantic( 
      ({objects, context}) => context.marker == 'earn',
      ({objects, context, s}) => { 
        context.isResponse = true;
        context.inner = s(context.inner) 
      }
    )
    const s = new Semantics([s1, s2]);
    const objects = {}
    const context = { marker: "earn", inner: { marker: "time" } }
    const cs = s.apply(objects, context);
    expect(cs).toStrictEqual({ marker: 'earn', isResponse: true, inner: { marker: "time", isResponse: true } });
  });


  it('apply recursive', () => {
    const contexts = [ { "action": "move", "marker": "move", "thing": {"pullFromContext": true, "language": "english", "marker": "anyConcept", "word": "it"}, "place": {"id": "building1", "language": "english", "marker": "buildingConcept", "word": "building1"}}]; 
    const s= new Semantics([
            new Semantic( 
              ({objects, context}) => context.pullFromContext,
              ({objects, context}) => { 
                object = objects.mentioned[0] 
                Object.assign(context, object) 
                delete context.pullFromContext 
              }
            )])
    const objects = {"mentioned": [{"marker": "tankConcept", "word": "tank1"}]}
    const cs = s.apply(objects, contexts);
    expected = [{"action": "move", "marker": "move", "place": {"id": "building1", "language": "english", "marker": "buildingConcept", "word": "building1"}, "thing": {"language": "english", "marker": "tankConcept", "word": "tank1"}}]
    expect(cs).toStrictEqual(expected)
  });
})
